﻿using System.Reflection;
using FluentValidation;
using FluentValidation.AspNetCore;
using MediatR.NotificationPublishers;
using Serilog;
using Sharing.Application;
using Sharing.Province;
using Sharing.Province.Handlers.HuiJu;
using Sharing.Repository.Extensions;
using XF.Domain.Dependency;
using XF.Domain.Filters;
using XF.EasyCaching;
using XF.Utility.MQ;

namespace Sharing.Api;

internal static class StartupExtensions
{
    const string CorsOrigins = "CorsOrigins";
    internal static WebApplication ConfigureServices(this WebApplicationBuilder builder)
    {
        var services = builder.Services;
        var configuration = builder.Configuration;

        services.AddHttpContextAccessor();

        services.Configure<ChannelConfiguration>(d => configuration.GetSection(nameof(ChannelConfiguration)).Bind(d));

        // Add services to the container.
        services
            .BatchInjectServices()
            .AddSharingProvince()
            .AddApplication()
            ;

        //Authentication
        services.RegisterAuthentication(configuration);

        services.AddControllers(options =>
        {
            options.Filters.Add<UserFriendlyExceptionFilter>();
        })
            .AddApplicationPart()
            //            .AddDapr(d =>
            //            {
            //#if DEBUG
            //                d.UseHttpEndpoint("http://192.168.100.223:50112");
            //#endif
            //            })
            ;

        // Learn more about configuring Swagger/OpenAPI at https://aka.ms/aspnetcore/swashbuckle
        services.AddEndpointsApiExplorer();

        services.AddHttpClient();

        //swagger
        services.RegisterSwagger();

        /* CORS */
        services.RegisterCors(configuration, CorsOrigins);

        //mapster
        services.RegisterMapper();

        //mediatr
        services.AddMediatR(cfg =>
        {
            cfg.RegisterServicesFromAssembly(typeof(StartupExtensions).Assembly);
            cfg.NotificationPublisher = new TaskWhenAllPublisher(); // this will be singleton
            cfg.NotificationPublisherType = typeof(TaskWhenAllPublisher); // this will be the ServiceLifetime
            cfg.RegisterProvinceServices();
        });

        //sqlsugar
        services.AddSqlSugar(configuration, "Sharing");

        //cache
        services.AddCache(d =>
        {
            d.ConnectionString = configuration.GetConnectionString("Redis") ?? string.Empty;
            d.Prefix = "Sharing";
            d.TopicName = "sharing-topic";
            d.Database = 2;
        });

        //validator
        services.AddFluentValidationAutoValidation(config =>
        {
            config.DisableDataAnnotationsValidation = true;
        })
            .AddValidatorsFromAssembly(typeof(Program).Assembly);

        //mq
        services.AddMq(configuration);

        return builder.Build();
    }

    internal static WebApplication ConfigurePipelines(this WebApplication app)
    {
        app.UseSerilogRequestLogging();

        //var useExless = app.Configuration.GetSection("Exceptionless:InUse").Get<bool>();
        //if (useExless)
        //{
        //    app.UseExceptionless();
        //}

        var swaggerEnable = app.Configuration.GetSection("Swagger").Get<bool>();
        // Configure the HTTP request pipeline.
        if (swaggerEnable)
        {
            app.UseSwagger();
            app.UseSwaggerUI();
        }

        app.UseCors(CorsOrigins);

        app.UseAuthentication();
        app.UseAuthorization();

        //app.UseCloudEvents();
        app.MapControllers()
            .RequireAuthorization()
            ;
        //app.MapSubscribeHandler();

        return app;
    }
}